#!/bin/bash

set -e

# Detect the operating system
OS=$(awk -F= '/^NAME/{print $2}' /etc/os-release)

# Install Ruby, Nginx, and Git
case $OS in
  "CentOS Linux"|"Fedora")
    sudo yum install -y ruby nginx git
    ;;
  "Arch Linux")
    sudo pacman -Syu --noconfirm ruby nginx git
    ;;
  "Debian GNU/Linux"|"Ubuntu"|"Raspbian GNU/Linux")
    sudo apt-get update
    sudo apt-get install -y ruby nginx git
    ;;
  *)
    echo "Unsupported operating system: $OS"
    exit 1
    ;;
esac

# Install the 'webrick' gem
if ! gem list webrick -i > /dev/null; then
    echo "Installing webrick..."
    gem install webrick
fi


echo "Enabling Nginx service..."
sudo systemctl enable nginx
sudo systemctl start nginx

# Make a directory in the current user's home folder called 'ruby-proxy'
if [ ! -d ~/ruby-proxy ]; then
    echo "Creating directory 'ruby-proxy'..."
    mkdir ~/ruby-proxy
fi

# Clone the contents of the specified git repository into that folder
if [ ! -d ~/ruby-proxy/ruby-rewrite-proxy ]; then
    echo "Cloning repository..."
    git clone https://adastra7.net/git/crt/ruby-rewrite-proxy.git ~/ruby-proxy
fi

# Ask the user to set server_name and proxy_set_header
read -p "Enter server_name (the URL from which you will access the Website): " server_name
read -p "Enter proxy_set_header (the host header for the website you wanna modify/proxy): " proxy_set_header

# Create the Nginx config file
if [ ! -f /etc/nginx/sites-available/ruby-proxy ]; then
    echo "Creating Nginx configuration file..."
    cat << EOF > /etc/nginx/sites-available/ruby-proxy
resolver 8.8.8.8;

server {
    listen 80;
    server_name $server_name;
    
    location / {
        proxy_pass http://localhost:8000/;
        proxy_set_header Host $proxy_set_header;
        proxy_set_header X-Real-IP \$remote_addr;
        proxy_set_header X-Forwarded-For \$proxy_add_x_forwarded_for;
        proxy_ssl_server_name on;
        proxy_ssl_verify off;
    }

    location /favicon.ico {
        return 204;
        access_log off;
        log_not_found off;
    }

    location /robots.txt {
        return 200 "User-agent: *\nDisallow: /";
        access_log off;
        log_not_found off;
    }
}
EOF
fi

# Enable the site
if [ ! -f /etc/nginx/sites-enabled/ruby-proxy ]; then
    echo "Enabling the site..."
    sudo ln -s /etc/nginx/sites-available/ruby-proxy /etc/nginx/sites-enabled/
fi

# Reload Nginx to apply the changes
echo "Reloading Nginx..."
sudo systemctl reload nginx

echo "The script has completed. Please edit the config.yml in ~/ruby-proxy/config.yml to your liking now before running proxy.rb to start the proxy"

